searchData={"content_type":"text/markdown","items":[{"doc":"Skema is a simple schema validation and casting library for Elixir.\n\nProvides four main APIs:\n1. `cast_and_validate/2` - casting and validating data with given schema\n2. `cast/2` - casting data with given schema\n3. `validate/2` - validating data with given schema\n4. `transform/2` - transforming data with given schema","ref":"Skema.html","title":"Skema","type":"module"},{"doc":"Skema schema can be a map with field name as key and field definition as value,\nor a schema module.\n\n```elixir\nschema = %{\n  email: [type: :string, required: true],\n  age: [type: :integer, number: [min: 18]],\n  hobbies: [type: {:array, :string}]\n}\n```\n\nor using defschema:\n\n```elixir\ndefmodule UserSchema do\n  use Skema\n\n  defschema do\n    field :email, :string, required: true\n    field :age, :integer, number: [min: 18]\n    field :hobbies, {:array, :string}\n  end\nend\n```","ref":"Skema.html#module-define-schema","title":"Define schema - Skema","type":"module"},{"doc":"Skema provides a complete data processing pipeline:\n\n1. **Cast** - Convert raw input to proper types\n2. **Validate** - Check business rules and constraints\n3. **Transform** - Normalize, format, and compute derived values\n\n```elixir\n# Full pipeline\nraw_data = %{\"email\" => \"  JOHN@EXAMPLE.COM  \", \"age\" => \"25\"}\n\nwith {:ok, cast_data} <- Skema.cast(raw_data, schema),\n     :ok <- Skema.validate(cast_data, schema),\n     {:ok, final_data} <- Skema.transform(cast_data, transform_schema) do\n  {:ok, final_data}\nend\n\n# Or use the combined function\ncase Skema.cast_and_validate(raw_data, schema) do\n  {:ok, data} -> IO.puts(\"Data is valid\")\n  {:error, errors} -> IO.puts(inspect(errors))\nend\n```","ref":"Skema.html#module-data-processing-pipeline","title":"Data Processing Pipeline - Skema","type":"module"},{"doc":"Transform allows you to modify data after casting and validation:\n\n```elixir\ntransform_schema = %{\n  email: [into: &String.downcase/1],\n  full_name: [\n    into: fn _value, data ->\n      \"#{data.first_name} #{data.last_name}\"\n    end\n  ],\n  user_id: [as: :id, into: &generate_uuid/1]\n}\n```","ref":"Skema.html#module-transformation-features","title":"Transformation Features - Skema","type":"module"},{"doc":"- `into` - Function to transform the field value\n- `as` - Rename the field in the output","ref":"Skema.html#module-transformation-options","title":"Transformation Options - Skema","type":"module"},{"doc":"```elixir\n# Simple transformation\n[into: &String.upcase/1]\n\n# Access to all data\n[into: fn value, data -> transform_with_context(value, data) end]\n\n# Module function\n[into: {MyModule, :transform_field}]\n\n# Error handling\n[into: fn value ->\n  if valid?(value) do\n    {:ok, normalize(value)}\n  else\n    {:error, \"invalid value\"}\n  end\nend]\n```","ref":"Skema.html#module-function-types","title":"Function Types - Skema","type":"module"},{"doc":"- **cast/2** - Type conversion only, returns `{:ok, data}` or `{:error, result}`\n- **validate/2** - Rule checking only, returns `:ok` or `{:error, result}`\n- **transform/2** - Data transformation, returns `{:ok, data}` or `{:error, result}`\n- **cast_and_validate/2** - Combined cast + validate, returns `{:ok, data}` or `{:error, errors}`","ref":"Skema.html#module-api-differences","title":"API Differences - Skema","type":"module"},{"doc":"Cast data to proper types according to schema.\n\nReturns `{:ok, data}` if casting succeeds, `{:error, result}` otherwise.","ref":"Skema.html#cast/2","title":"Skema.cast/2","type":"function"},{"doc":"Cast and validate data with given schema.\n\nReturns `{:ok, data}` if both casting and validation succeed,\n`{:error, errors}` otherwise.","ref":"Skema.html#cast_and_validate/2","title":"Skema.cast_and_validate/2","type":"function"},{"doc":"Transform data according to schema transformation rules.\n\nReturns `{:ok, data}` if transformation succeeds, `{:error, result}` otherwise.","ref":"Skema.html#transform/2","title":"Skema.transform/2","type":"function"},{"doc":"Validate data according to schema rules.\n\nReturns `:ok` if validation succeeds, `{:error, result}` otherwise.","ref":"Skema.html#validate/2","title":"Skema.validate/2","type":"function"},{"doc":"Result Struct for Skema operations","ref":"Skema.Result.html","title":"Skema.Result","type":"module"},{"doc":"Get error from result for given field.","ref":"Skema.Result.html#get_error/2","title":"Skema.Result.get_error/2","type":"function"},{"doc":"Create a new Result struct with given schema map and params.","ref":"Skema.Result.html#new/1","title":"Skema.Result.new/1","type":"function"},{"doc":"Put valid data to result.","ref":"Skema.Result.html#put_data/3","title":"Skema.Result.put_data/3","type":"function"},{"doc":"Put error to result.","ref":"Skema.Result.html#put_error/3","title":"Skema.Result.put_error/3","type":"function"},{"doc":"Schema definition macros for Skema.\n\nProvides `defschema` macro for defining structured schemas with type annotations,\nvalidation rules, and automatic struct generation.","ref":"Skema.Schema.html","title":"Skema.Schema","type":"module"},{"doc":"- Type-safe struct generation with `@type` annotations\n- Automatic field validation and casting\n- Support for default values (static or function-based)\n- Required field enforcement via `@enforce_keys`\n- Nested schema support\n- Custom casting and validation functions","ref":"Skema.Schema.html#module-features","title":"Features - Skema.Schema","type":"module"},{"doc":"```elixir\ndefmodule User do\n  use Skema.Schema\n\n  defschema do\n    field :name, :string, required: true\n    field :email, :string, required: true\n    field :age, :integer, default: 0\n    field :created_at, :naive_datetime, default: &NaiveDateTime.utc_now/0\n  end\nend\n```","ref":"Skema.Schema.html#module-basic-usage","title":"Basic Usage - Skema.Schema","type":"module"},{"doc":"```elixir\ndefmodule BlogPost do\n  use Skema.Schema\n\n  defschema do\n    field :title, :string, required: true, length: [min: 5, max: 100]\n    field :content, :string, required: true\n    field :status, :string, default: \"draft\", in: ~w(draft published archived)\n    field :tags, {:array, :string}, default: []\n    field :author, User, required: true\n    field :published_at, :naive_datetime,\n      default: fn -> if status == \"published\", do: NaiveDateTime.utc_now() end\n  end\nend\n```","ref":"Skema.Schema.html#module-advanced-usage","title":"Advanced Usage - Skema.Schema","type":"module"},{"doc":"```elixir\ndefmodule Company do\n  use Skema.Schema\n\n  defschema Address do\n    field :street, :string, required: true\n    field :city, :string, required: true\n    field :country, :string, default: \"US\"\n  end\n\n  defschema do\n    field :name, :string, required: true\n    field :address, Address, required: true\n    field :employees, {:array, User}, default: []\n  end\nend\n```","ref":"Skema.Schema.html#module-nested-schemas","title":"Nested Schemas - Skema.Schema","type":"module"},{"doc":"Defines a typed struct with validation capabilities.\n\nInside a `defschema` block, each field is defined through the `field/3` macro.","ref":"Skema.Schema.html#defschema/2","title":"Skema.Schema.defschema/2","type":"macro"},{"doc":"- `type` - The field type (built-in Elixir types, custom types, or other schemas)\n- `required` - If `true`, the field is required and cannot be `nil`\n- `default` - Default value (static value or zero-arity function)\n- `cast_func` - Custom casting function\n- `from` - Source field name if different from target field name\n- `as` - Target field name if different from source field name\n- `into` - Transformation function for `Skema.transform/2`\n- Validation options: `length`, `number`, `format`, `in`, `not_in`, `func`, `each`","ref":"Skema.Schema.html#defschema/2-options-per-field","title":"Options per field - Skema.Schema.defschema/2","type":"macro"},{"doc":"","ref":"Skema.Schema.html#defschema/2-examples","title":"Examples - Skema.Schema.defschema/2","type":"macro"},{"doc":"```elixir\ndefmodule Person do\n  use Skema.Schema\n\n  defschema do\n    field :name, :string, required: true\n    field :age, :integer, number: [min: 0, max: 150]\n    field :email, :string, format: ~r/@/\n  end\nend\n```","ref":"Skema.Schema.html#defschema/2-basic-schema","title":"Basic Schema - Skema.Schema.defschema/2","type":"macro"},{"doc":"```elixir\ndefmodule MyModule do\n  use Skema.Schema\n\n  defschema User do\n    field :username, :string, required: true\n    field :role, :string, default: \"user\", in: ~w(user admin)\n  end\n\n  defschema Post do\n    field :title, :string, required: true\n    field :author, User, required: true\n    field :tags, {:array, :string}, default: []\n  end\nend\n```","ref":"Skema.Schema.html#defschema/2-schema-in-submodule","title":"Schema in Submodule - Skema.Schema.defschema/2","type":"macro"},{"doc":"```elixir\ndefschema Document do\n  field :title, :string, required: true\n  field :created_at, :naive_datetime, default: &NaiveDateTime.utc_now/0\n  field :uuid, :string, default: fn -> Ecto.UUID.generate() end\nend\n```","ref":"Skema.Schema.html#defschema/2-dynamic-defaults","title":"Dynamic Defaults - Skema.Schema.defschema/2","type":"macro"},{"doc":"Defines a field in a typed struct.","ref":"Skema.Schema.html#field/3","title":"Skema.Schema.field/3","type":"macro"},{"doc":"# Basic field\n    field :name, :string\n\n    # Required field\n    field :email, :string, required: true\n\n    # Field with default value\n    field :status, :string, default: \"active\"\n\n    # Field with validation\n    field :age, :integer, number: [min: 0, max: 150]\n\n    # Field with custom casting\n    field :tags, {:array, :string}, cast_func: &parse_comma_separated/1","ref":"Skema.Schema.html#field/3-examples","title":"Examples - Skema.Schema.field/3","type":"macro"},{"doc":"* `default` - Sets the default value for the field (can be a value or function)\n  * `required` - If set to true, enforces the field and makes its type non-nullable\n  * `cast_func` - Custom function for casting the field value\n  * `from` - Use value from a different source field name\n  * `as` - Output the field with a different name\n  * `into` - Transform function for post-processing\n  * Validation options: `length`, `number`, `format`, `in`, `not_in`, `func`, `each`","ref":"Skema.Schema.html#field/3-options","title":"Options - Skema.Schema.field/3","type":"macro"},{"doc":"Cast data to target type.\nMuch code of this module is borrowed from `Ecto.Type`","ref":"Skema.Type.html","title":"Skema.Type","type":"module"},{"doc":"","ref":"Skema.Type.html#cast/2","title":"Skema.Type.cast/2","type":"function"},{"doc":"","ref":"Skema.Type.html#cast!/2","title":"Skema.Type.cast!/2","type":"function"},{"doc":"# Skema\n\nPhoenix request params validation library.\n\n[![Build Status](https://github.com/bluzky/skema/workflows/Elixir%20CI/badge.svg)](https://github.com/bluzky/skema/actions) [![Coverage Status](https://coveralls.io/repos/github/bluzky/skema/badge.svg?branch=main)](https://coveralls.io/github/bluzky/skema?branch=main) [![Hex Version](https://img.shields.io/hexpm/v/skema.svg)](https://hex.pm/packages/skema) [![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/skema/)\n\n- [Skema](#skema)\n    - [Why Skema](#why-skema)\n    - [Installation](#installation)\n    - [Usage](#usage)\n    - [Define schema](#define-schema)\n        - [Default value](#default-value)\n        - [Custom cast function](#custom-cast-function)\n        - [Nested schema](#nested-schema)\n        - [Transform data](#transform-data)\n    - [Validation](#validation)\n    - [Data Processing Pipeline](#data-processing-pipeline)\n    - [Contributors](#contributors)","ref":"readme.html","title":"Skema","type":"extras"},{"doc":"- Reduce code boilerplate\n- Shorter schema definition\n- Default function which generate value each casting time\n- Custom validation functions\n- Custom cast functions\n- Data transformation and normalization\n- Complete data processing pipeline (cast → validate → transform)","ref":"readme.html#why-skema","title":"Why Skema - Skema","type":"extras"},{"doc":"[Available in Hex](https://hex.pm/skema), the package can be installed\nby adding `skema` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:skema, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Skema","type":"extras"},{"doc":"**Process order**\n> Cast data → validate casted data → transform data\n\n```elixir\nuse Skema\n\ndefschema IndexParams do\n    field :keyword, :string\n    field :status, :string, required: true\n    field :group_id, :integer, number: [greater_than: 0]\n    field :name, :string\nend\n\ndef index(conn, params) do\n    with {:ok, better_params} <- Skema.cast_and_validate(params, IndexParams) do\n        # do anything with your params\n    else\n        {:error, errors} -> # return params error\n    end\nend\n```","ref":"readme.html#usage","title":"Usage - Skema","type":"extras"},{"doc":"```elixir\nuse Skema\n\ndefschema IndexParams do\n    field :keyword, :string\n    field :status, :string, required: true\n    field :group_id, :integer, number: [greater_than: 0]\n    field :name, :string\nend\n```\n\nDefine a field using macro `field(field_name :: atom(), type :: term(), opts \\\\ [])`\n\n- `type`: `Skema` supports same data types as `Ecto`. Code borrowed from Ecto\n\nSupported options:\n\n- `default`: default value or default function\n- `cast_func`: custom cast function\n- `number, format, length, in, not_in, func, required, each` are available validations\n- `from`: use value from another field\n- `as`: alias key you will receive from `Skema.cast` if casting is succeeded\n- `into`: transformation function (used by `Skema.transform/2`)","ref":"readme.html#define-schema","title":"Define schema - Skema","type":"extras"},{"doc":"You can define a default value for a field if it's missing from the params.\n\n```elixir\nfield :status, :string, default: \"pending\"\n```\n\nOr you can define a default value as a function. This function is evaluated when `Skema.cast` gets invoked.\n\n```elixir\nfield :date, :utc_datetime, default: &DateTime.utc_now/0\n```","ref":"readme.html#default-value","title":"Default value - Skema","type":"extras"},{"doc":"You can define your own casting function using the `cast_func` option.\nYour `cast_func` must follow this spec:\n\n```elixir\nfn(any) :: {:ok, any} | {:error, binary} | :error\n```\n\n#### Simple cast function\n\n```elixir\ndef my_array_parser(value) do\n    if is_binary(value) do\n        ids =\n            String.split(value, \",\")\n            |> Enum.map(&String.to_integer/1)\n\n        {:ok, ids}\n    else\n        {:error, \"Invalid string\"}\n    end\nend\n\ndefschema Sample do\n   field :user_ids, {:array, :integer}, cast_func: &my_array_parser/1\nend\n```\n\n#### Cast function with data access\n\n```elixir\ndefschema UserParams do\n   field :full_name, :string, cast_func: fn _value, data ->\n       {:ok, \"#{data.first_name} #{data.last_name}\"}\n   end\nend\n```\n\n#### Module function tuple\n\n```elixir\ndefschema Sample do\n   field :email, :string, cast_func: {MyModule, :normalize_email}\nend\n```","ref":"readme.html#custom-cast-function","title":"Custom cast function - Skema","type":"extras"},{"doc":"With `Skema` you can parse and validate nested maps and lists easily\n\n```elixir\ndefschema Address do\n    field :street, :string\n    field :district, :string\n    field :city, :string\nend\n\ndefschema User do\n    field :name, :string\n    field :email, :string, required: true\n    field :addresses, {:array, Address}\nend\n```\n\n#### ⚠️ Important: Schema Definition Order\n\nWhen using `defschema` with nested schemas, **always define the nested schemas before the schemas that reference them**. This is a compile-time dependency requirement.\n\n**❌ Wrong - Will fail with \"is invalid\" error:**\n```elixir\ndefschema HTTPRequestSchema do\n  field :auth, AuthSchema  # ← AuthSchema not yet defined!\nend\n\ndefschema AuthSchema do\n  field :type, :string, required: true\nend\n```\n\n**✅ Correct - Define dependencies first:**\n```elixir\n# Define AuthSchema FIRST\ndefschema AuthSchema do\n  field :type, :string, required: true\n  field :token, :string\nend\n\n# Then define schemas that reference it\ndefschema HTTPRequestSchema do\n  field :url, :string, required: true\n  field :auth, AuthSchema  # ← Now AuthSchema exists!\nend\n```","ref":"readme.html#nested-schema","title":"Nested schema - Skema","type":"extras"},{"doc":"Transform allows you to modify and normalize data after casting and validation:\n\n```elixir\n# Using schema definition\ndefschema UserParams do\n    field :email, :string, into: &String.downcase/1\n    field :user_name, :string, as: :username, into: &String.trim/1\nend\n\n# Or using map schema with Skema.transform/2\ntransform_schema = %{\n    email: [into: &String.downcase/1],\n    full_name: [into: fn _value, data -> \"#{data.first_name} #{data.last_name}\" end],\n    tags: [into: fn tags_string ->\n        tags_string\n        |> String.split(\",\")\n        |> Enum.map(&String.trim/1)\n    end]\n}\n\n{:ok, transformed_data} = Skema.transform(data, transform_schema)\n```\n\n#### Transform function types\n\n```elixir\n# Simple transformation\nfield :name, :string, into: &String.upcase/1\n\n# Access to all data\nfield :display_name, :string, into: fn _value, data ->\n    \"#{data.title} #{data.name}\"\nend\n\n# Module function\nfield :email, :string, into: {MyModule, :normalize_email}\n\n# Error handling\nfield :score, :integer, into: fn value ->\n    if value < 0 do\n        {:error, \"score cannot be negative\"}\n    else\n        {:ok, value}\n    end\nend\n\n# Field renaming\nfield :user_email, :string, as: :email, into: &String.downcase/1\n\n# Partial application with predefined parameters\nfield :name_parts, {:array, :string}, into: &String.split(&1, \" \", parts: 2)\n```","ref":"readme.html#transform-data","title":"Transform data - Skema","type":"extras"},{"doc":"`Skema` uses `Valdi` validation library. You can read more about [Valdi here](https://github.com/bluzky/valdi)\n\nBasically it supports following validations:\n\n- validate inclusion/exclusion\n- validate length for string and enumerable types\n- validate number\n- validate string format/pattern\n- validate custom function\n- validate required(not nil) or not\n- validate each array item\n\n```elixir\ndefschema Product do\n    field :sku, :string, required: true, length: [min: 6, max: 20]\n    field :name, :string, required: true\n    field :quantity, :integer, number: [min: 0]\n    field :type, :string, in: ~w(physical digital)\n    field :expiration_date, :naive_datetime, func: &my_validation_func/1\n    field :tags, {:array, :string}, each: [length: [max: 50]]\nend\n```","ref":"readme.html#validation","title":"Validation - Skema","type":"extras"},{"doc":"- Can accept function or `{module, function}` tuple\n- Only supports 2-arity functions\n\n```elixir\ndef require_email?(value, data), do: is_nil(data.phone)\n\n# ...\n\nfield :email, :string, required: {__MODULE__, :require_email?}\n```","ref":"readme.html#dynamic-required","title":"Dynamic required - Skema","type":"extras"},{"doc":"Support validating array items with `:each` option. `each` accepts a list of validators:\n\n```elixir\nfield :values, {:array, :number}, each: [number: [min: 20, max: 50]]\n```","ref":"readme.html#validate-array-items","title":"Validate array items - Skema","type":"extras"},{"doc":"Skema provides a complete data processing pipeline with four main functions:","ref":"readme.html#data-processing-pipeline","title":"Data Processing Pipeline - Skema","type":"extras"},{"doc":"Converts raw input data to proper types according to schema definitions.\n\n```elixir\nschema = %{age: :integer, active: :boolean}\n{:ok, %{age: 25, active: true}} = Skema.cast(%{\"age\" => \"25\", \"active\" => \"true\"}, schema)\n```","ref":"readme.html#1-skema-cast-2-type-conversion","title":"1. Skema.cast/2 - Type Conversion - Skema","type":"extras"},{"doc":"Validates data against business rules and constraints.\n\n```elixir\nschema = %{age: [type: :integer, number: [min: 18]]}\n:ok = Skema.validate(%{age: 25}, schema)\n{:error, _} = Skema.validate(%{age: 15}, schema)\n```","ref":"readme.html#2-skema-validate-2-rule-checking","title":"2. Skema.validate/2 - Rule Checking - Skema","type":"extras"},{"doc":"Normalizes, formats, and computes derived values.\n\n```elixir\ntransform_schema = %{\n  email: [into: &String.downcase/1],\n  full_name: [into: fn _value, data -> \"#{data.first_name} #{data.last_name}\" end]\n}\n\n{:ok, transformed} = Skema.transform(data, transform_schema)\n```","ref":"readme.html#3-skema-transform-2-data-transformation","title":"3. Skema.transform/2 - Data Transformation - Skema","type":"extras"},{"doc":"Performs casting and validation in one step.\n\n```elixir\n{:ok, data} = Skema.cast_and_validate(params, schema)\n```","ref":"readme.html#4-skema-cast_and_validate-2-combined-operation","title":"4. Skema.cast_and_validate/2 - Combined Operation - Skema","type":"extras"},{"doc":"```elixir\n# Raw input\nraw_params = %{\n  \"email\" => \"  JOHN@EXAMPLE.COM  \",\n  \"age\" => \"25\",\n  \"first_name\" => \"john\",\n  \"last_name\" => \"doe\"\n}\n\n# Schema definition\nschema = %{\n  email: [type: :string, required: true],\n  age: [type: :integer, number: [min: 18]],\n  first_name: [type: :string, required: true],\n  last_name: [type: :string, required: true]\n}\n\n# Transform schema\ntransform_schema = %{\n  email: [into: fn email -> String.trim(email) |> String.downcase() end],\n  full_name: [into: fn _value, data -> \"#{data.first_name} #{data.last_name}\" end],\n  first_name: [into: &String.capitalize/1],\n  last_name: [into: &String.capitalize/1]\n}\n\n# Step by step\nwith {:ok, cast_data} <- Skema.cast(raw_params, schema),\n     :ok <- Skema.validate(cast_data, schema),\n     {:ok, final_data} <- Skema.transform(cast_data, transform_schema) do\n  {:ok, final_data}\n  # Result: %{\n  #   email: \"john@example.com\",\n  #   age: 25,\n  #   first_name: \"John\",\n  #   last_name: \"Doe\",\n  #   full_name: \"John Doe\"\n  # }\nend\n```","ref":"readme.html#complete-pipeline-example","title":"Complete Pipeline Example - Skema","type":"extras"},{"doc":"If you find a bug or want to improve something, please send a pull request. Thank you!","ref":"readme.html#thank-you","title":"Thank you - Skema","type":"extras"}],"producer":{"name":"ex_doc","version":"0.38.2"},"proglang":"elixir"}